/**
 * 
 */
package processView;

import java.awt.Color;
import java.awt.Component;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.TexturePaint;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.util.Observable;
import java.util.Observer;

import javax.swing.Action;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.event.MouseInputListener;
import javax.swing.KeyStroke;

import javax.swing.AbstractAction;

import processModel.Process;
import utilities.ActionEnum;
import utilities.AppConstants;

/**
 * ProcessDiagram is one of the ContceteObservers participating in
 * our implementation of the Observer Design Pattern. A ProcessDiagram object 
 * observes a Process object, changes its state and querries it. Since Observer
 * Design Pattern is closely related to the Model-View-Controller System 
 * architecture, this class implements the View component of the applet.
 * ProcessApplet class implements Controller component. Process class implements 
 * Model component.
 * 
 * @author Kon Zakharov, k.zakharov@cosc.canterbury.ac.nz
 */
public class ProcessDiagram extends JPanel implements Observer,MouseInputListener {
	/**
	 * Required and autogenerated by Eclipse.
	 */
	private static final long serialVersionUID = 5750316863195015532L;
	/**
	 * The Subject participant of the Observer Design Pattern.
	 */
	private Process subject;
	
	public Action deleteAction = new DeleteAction();
		
	/**
	 */
	public ProcessDiagram(Process subj) {
		// No layout, isDoubleBuffered.
		super(null, true);
		subject = subj;
		setOpaque(true);
		addMouseMotionListener(this);
        addMouseListener(this);
        getInputMap().put(KeyStroke.getKeyStroke(AppConstants.DELETE), AppConstants.DELETE);
        getActionMap().put(AppConstants.DELETE, deleteAction);
	}
	
	public void paintComponent(Graphics g) {
        super.paintComponent(g);
        
        Graphics2D g2 = (Graphics2D)g;
        // Draw the grid.
        g2.setPaint(getGridTexture(AppConstants.GRID_SIZE, AppConstants.GRID_BG_COLOR, AppConstants.GRID_COLOR));
        int w = getSize().width;
        int h = getSize().height;
        g2.fill(new Rectangle(0, 0, w, h));
        
        // We are using remainder division to make sure the origin appears 
        // on the intersection of the grid lines.
        int originX = AppConstants.GRID_SIZE * 2;
        int originY = this.getHeight() - (this.getHeight() % AppConstants.GRID_SIZE)  - AppConstants.GRID_SIZE * 2;
        Point2D originPoint = new Point2D.Double(originX, originY);
        int extremeX = this.getWidth() - (this.getWidth() % AppConstants.GRID_SIZE) - AppConstants.GRID_SIZE *2;
        Point2D extremeXPoint = new Point2D.Double(extremeX, originY);
        int extremeY = AppConstants.GRID_SIZE * 2;
        Point2D extremeYPoint = new Point2D.Double(originX, extremeY);
        
        // Avoid jaggies.
        g2.setRenderingHints(AppConstants.ANTIALIAS_Hint);
        // Draw the coordinate system.
        g2.setColor(Color.BLACK);
        g2.draw(new Line2D.Double(originPoint, extremeXPoint));
        g2.draw(new Line2D.Double(originPoint, extremeYPoint));
        
        // Now draw the arrow heads.
        g2.draw(new Line2D.Double(extremeXPoint.getX(), extremeXPoint.getY(), 
        		extremeXPoint.getX() - AppConstants.ARROW_HEAD_SIZE, 
        		extremeXPoint.getY() + AppConstants.ARROW_HEAD_SIZE * .3));
        g2.draw(new Line2D.Double(extremeXPoint.getX(), extremeXPoint.getY(), 
        		extremeXPoint.getX() - AppConstants.ARROW_HEAD_SIZE, 
        		extremeXPoint.getY() - AppConstants.ARROW_HEAD_SIZE * .3));
        g2.draw(new Line2D.Double(extremeYPoint.getX(), extremeYPoint.getY(), 
        		extremeYPoint.getX() - AppConstants.ARROW_HEAD_SIZE * .3, 
        		extremeYPoint.getY() + AppConstants.ARROW_HEAD_SIZE));
        g2.draw(new Line2D.Double(extremeYPoint.getX(), extremeYPoint.getY(), 
        		extremeYPoint.getX() + AppConstants.ARROW_HEAD_SIZE * .3, 
        		extremeYPoint.getY() + AppConstants.ARROW_HEAD_SIZE));
        
        // Now draw the axis labels.
        g2.setFont(AppConstants.ARIAL_FONT);
        g2.drawString("V", (int)extremeXPoint.getX() - AppConstants.ARROW_HEAD_SIZE, 
        		(int)(extremeXPoint.getY() + AppConstants.ARROW_HEAD_SIZE * 1.5));
        g2.drawString("0", (int)originPoint.getX() - AppConstants.ARROW_HEAD_SIZE, 
        		(int)(originPoint.getY() + AppConstants.ARROW_HEAD_SIZE));
        g2.drawString("P", (int)(extremeYPoint.getX() - AppConstants.ARROW_HEAD_SIZE * 1.3), 
        		(int)extremeYPoint.getY() + AppConstants.ARROW_HEAD_SIZE);
        
        // Now draw the process components.
        ((Process)subject).paint(g2);
	}
	
	/**
	 * Creates a square grid texture with the given dimensions, background and foreground colors.
	 * @param size required square grid dimensions.
	 * @param bg background color.
	 * @param fg foreground color.
	 * @return grid texture.
	 */
	private TexturePaint getGridTexture(int size, Color bg, Color fg) {
        BufferedImage bi = new BufferedImage(size, size, BufferedImage.TYPE_INT_RGB);
        Graphics2D tG2 = bi.createGraphics();
        tG2.setBackground(bg);
        tG2.clearRect(0, 0, size, size);
        tG2.setColor(fg);
        tG2.drawLine(size - 1, 0, size - 1, size);
        tG2.drawLine(0, size - 1, size, size - 1);
        Rectangle r = new Rectangle(0, 0, size, size);
        return new TexturePaint(bi, r);
    }
	
	public void disableDiagram() {
		this.setEnabled(false);
	}
		
	
	/**
	 *  @see java.util.Observer#update(java.util.Observable, java.lang.Object)
	 */
	public void update(Observable o, Object arg) {
		System.out.println("update in Process: " + arg + " from " + o);
		repaint();
	}

	public void mouseClicked(MouseEvent e) {
		System.err.println("mouseClicked " + e);
		//if (e.getButton() == MouseEvent.BUTTON1){
			//subject.setState(e.getPoint(), MouseEvent.MOUSE_CLICKED);
			repaint();
		//} 
	}

	public void mousePressed(MouseEvent e) {
		System.err.println("mousePressed " + e);
		if (SwingUtilities.isLeftMouseButton(e)) {
			requestFocusInWindow();
		}
		subject.setState(e.getPoint(), MouseEvent.MOUSE_PRESSED);
		repaint();
	}

	public void mouseReleased(MouseEvent e) {
		System.err.println("mouseReleased " + e);
		Frame f =  findParentFrame();
		subject.setState(e.getPoint(), MouseEvent.MOUSE_RELEASED);
		subject.parentFrame(f);
		if (e.getButton() == MouseEvent.BUTTON1)			
			subject.setState(ActionEnum.EDIT_DONE);
		
		if (e.getButton() == MouseEvent.BUTTON3 || e.getClickCount() >= 2){
			//subject.setState(ActionEnum.RIGHT_CLICK_MENU);
			subject.setState(ActionEnum.EDIT_DONE);
			subject.startRightClickMenu(f, e.getPoint());
		}
		repaint();
	}

	public void mouseDragged(MouseEvent e) {
		System.out.println("Mouse_Dragged>>>>" + e.getPoint());
		subject.setState(e.getPoint(), MouseEvent.MOUSE_DRAGGED);
		repaint();
	}
	
	public void mouseEntered(MouseEvent e) {}

	public void mouseExited(MouseEvent e) {}

	public void mouseMoved(MouseEvent e) {}
	
	private Frame findParentFrame(){
	    Component c = getParent();

	    while(true){
	      if(c instanceof Frame)
	        return (Frame)c;
	      c = c.getParent();
	    }
	  }
	
	class DeleteAction extends AbstractAction {
		public void actionPerformed(ActionEvent e) {
			System.out.println("delete key");
			if(subject.selectedObject!=null){
				subject.setState(ActionEnum.DELETE); 
			}
		}
	}
}
